@@ -93,7 +93,7 @@ impl Downloader {
         path: &Path,
         mp: MultiProgress,
     ) -> Result<Option<(Response, ProgressBar)>> {
-        debug!("Starting download of {} to {:?}", url, path);
+        debug!("Starting download of {url} to {path:?}");
         let res = self.client.get(url);
         let res = if self.always_download {
             res
@@ -114,10 +114,7 @@ impl Downloader {
         if !self.always_download && res.status() == StatusCode::NOT_MODIFIED {
             // this will only trigger if always_download is not set and the server reports that the
             // file was not modified
-            debug!(
-                "Skipping {}: always_download is not set and the file was not modified",
-                url
-            );
+            debug!("Skipping {url}: always_download is not set and the file was not modified");
             return Ok(None);
         }
 
@@ -198,7 +195,7 @@ impl Downloader {
         fs::rename(&tmp_file, path).with_context(|| {
             format!("Failed to move temporary file '{tmp_file:?}' to '{path:?}'")
         })?;
-        debug!("Download of {} to {:?} done", url, path);
+        debug!("Download of {url} to {path:?} done");
         Ok(CacheState::FreshFiles)
     }
 }
@@ -260,7 +257,7 @@ pub(crate) struct Cache {
 impl Cache {
     pub async fn new(force_download: bool, archive_mirror: &str) -> Result<Self> {
         let mut cache = Self {
-            base_directory: BaseDirectories::with_prefix("Debian-RT-tools")?,
+            base_directory: BaseDirectories::with_prefix("Debian-RT-tools"),
             downloader: Downloader::new(force_download),
             archive_mirror: archive_mirror.into(),
             unstable: empty_release(),
@@ -371,7 +368,16 @@ impl Cache {
 
     fn release_urls(&self, suite: Suite) -> Vec<DownloadInfo> {
         vec![DownloadInfo::new(
-            format!("{}/dists/{}/Release", self.archive_mirror, suite).into(),
+            format!(
+                "{}/dists/{}/Release",
+                self.archive_mirror,
+                if suite == Suite::Stable(Some(Extension::ProposedUpdates)) {
+                    "proposed-updates".to_string()
+                } else {
+                    suite.to_string()
+                }
+            )
+            .into(),
             format!("Release_{suite}").into(),
         )]
     }
