@@ -11,15 +11,14 @@ use std::{
 use anyhow::{Context, Result};
 use assorted_debian_utils::{
     architectures::Architecture,
-    archive::{Codename, MultiArch, SuiteOrCodename},
-    package::PackageName,
+    archive::{MultiArch, SuiteOrCodename},
+    package::{PackageName, VersionedPackage},
     rfc822_like,
     version::PackageVersion,
     wb::{BinNMU, SourceSpecifier, WBCommandBuilder},
 };
 use async_trait::async_trait;
 use indicatif::{ProgressBar, ProgressBarIter, ProgressIterator};
-use itertools::Itertools;
 use log::{debug, error};
 use serde::Deserialize;
 
@@ -30,7 +29,7 @@ use crate::{
         Cache, CacheEntries, default_progress_style, default_progress_template, source_skip_binnmu,
     },
     source_packages,
-    udd_bugs::{UDDBugs, load_bugs_from_reader},
+    udd_bugs::UDDBugs,
     utils::execute_wb_commands,
 };
 
@@ -67,7 +66,7 @@ impl BinaryPackageParser {
 }
 
 impl Iterator for BinaryPackageParser {
-    type Item = (PackageName, Architecture, PackageVersion);
+    type Item = (PackageName, Architecture, PackageVersion, u32);
 
     fn next(&mut self) -> Option<Self::Item> {
         for binary_package in self.iterator.by_ref() {
@@ -80,11 +79,19 @@ impl Iterator for BinaryPackageParser {
                 continue;
             }
 
-            let (source_package, _) = binary_package.package.name_and_version();
+            let VersionedPackage {
+                package: source_package,
+                version,
+            } = binary_package.package.source_package();
             return Some((
-                source_package.clone(),
+                source_package,
                 binary_package.architecture,
-                binary_package.package.version,
+                version,
+                binary_package
+                    .package
+                    .version
+                    .binnmu_version()
+                    .unwrap_or_default(),
             ));
         }
 
@@ -113,76 +120,70 @@ impl<'a> NMUVersionSkew<'a> {
         }
     }
 
-    fn load_bugs(&self, codename: Codename) -> Result<UDDBugs> {
-        load_bugs_from_reader(
-            self.cache
-                .get_cache_bufreader(format!("udd-ftbfs-bugs-{codename}.yaml"))?,
-        )
-    }
-
     fn load_version_skew(
         &self,
         suite: SuiteOrCodename,
-    ) -> Result<Vec<(PackageName, PackageVersion, Vec<Architecture>)>> {
-        let ftbfs_bugs = self
-            .load_bugs(suite.into())
+    ) -> Result<Vec<(VersionedPackage, u32, Vec<Architecture>)>> {
+        let ftbfs_bugs = UDDBugs::load_for_codename(self.cache, suite)
             .with_context(|| format!("Failed to load bugs for {suite}"))?;
-        let mut packages: HashMap<PackageName, HashSet<(Architecture, PackageVersion)>> =
+        let mut packages: HashMap<PackageName, HashSet<(Architecture, PackageVersion, u32)>> =
             HashMap::new();
         for path in self.cache.get_package_paths(suite, false)? {
-            for (source, architecture, version) in BinaryPackageParser::new(path)? {
+            for (source, architecture, source_version, binnmu_version) in
+                BinaryPackageParser::new(path)?
+            {
                 // skip some packages that make no sense to binNMU
                 if source_skip_binnmu(source.as_ref()) {
                     continue;
                 }
 
                 if let Some(info) = packages.get_mut(&source) {
-                    info.insert((architecture, version));
+                    info.insert((architecture, source_version, binnmu_version));
                 } else {
                     packages.insert(source, {
                         let mut hs = HashSet::new();
-                        hs.insert((architecture, version));
+                        hs.insert((architecture, source_version, binnmu_version));
                         hs
                     });
                 }
             }
         }
 
-        let mut sources_architectures =
-            Vec::<(PackageName, PackageVersion, Vec<Architecture>)>::default();
-        for (source, source_info) in packages.into_iter().sorted_by_key(|value| value.0.clone()) {
-            let mut max_version_per_architecture: HashMap<Architecture, PackageVersion> =
+        let mut sources_architectures = vec![];
+        for (source, source_info) in packages.into_iter() {
+            let mut max_version_per_architecture: HashMap<Architecture, (PackageVersion, u32)> =
                 HashMap::default();
-            for (architecture, version) in &source_info {
+            for (architecture, source_version, binnmu_version) in &source_info {
                 if let Some(max_version) = max_version_per_architecture.get_mut(architecture) {
-                    if version > max_version {
-                        *max_version = version.clone();
+                    let combined_version = (source_version.clone(), *binnmu_version);
+                    if combined_version > *max_version {
+                        *max_version = combined_version;
                     }
                 } else {
-                    max_version_per_architecture.insert(*architecture, version.clone());
+                    max_version_per_architecture
+                        .insert(*architecture, (source_version.clone(), *binnmu_version));
                 }
             }
 
-            let all_versions: HashSet<_> = max_version_per_architecture.values().cloned().collect();
-            if all_versions.len() == 1 {
-                debug!("Skipping {}: package is in sync", source);
+            // check if the same source version is built on architectures
+            let all_versions_without_binnmu: HashSet<_> = max_version_per_architecture
+                .values()
+                .map(|(v, _)| v)
+                .collect();
+            if all_versions_without_binnmu.len() != 1 {
+                debug!("Skipping {source}: package is out-of-date on some architecture");
                 continue;
             }
 
-            let all_versions_without_binnmu: HashSet<_> = max_version_per_architecture
-                .values()
-                .map(|v| (*v).clone().without_binnmu_version())
-                .collect();
-            if all_versions_without_binnmu.len() != 1 {
-                debug!(
-                    "Skipping {}: package is out-of-date on some architecture",
-                    source
-                );
+            // check if the binNMU versions diverge
+            let all_versions: HashSet<_> = max_version_per_architecture.values().cloned().collect();
+            if all_versions.len() == 1 {
+                debug!("Skipping {source}: package is in sync");
                 continue;
             }
 
             // check if package FTBFS
-            if let Some(bugs) = ftbfs_bugs.bugs_for_source(source.as_ref()) {
+            if let Some(bugs) = ftbfs_bugs.bugs_for_source(&source) {
                 println!("# Skipping {source} due to FTBFS bugs ...");
                 for bug in bugs {
                     debug!(
@@ -198,13 +199,13 @@ impl<'a> NMUVersionSkew<'a> {
                 .max()
                 .map(|v| (*v).clone())
             else {
-                error!(
-                    "Skipping {}: package has no binaries in the archive",
-                    source
-                );
+                error!("Skipping {source}: package has no binaries in the archive");
                 continue;
             };
-            debug!("Max {} version: {}", source, max_version);
+            debug!(
+                "{}: max version: {} binNMU version: {}",
+                source, max_version.0, max_version.1
+            );
 
             let architectures = max_version_per_architecture
                 .iter()
@@ -216,9 +217,17 @@ impl<'a> NMUVersionSkew<'a> {
                     }
                 })
                 .collect();
-            sources_architectures.push((source, max_version, architectures));
+            sources_architectures.push((
+                VersionedPackage {
+                    package: source,
+                    version: max_version.0,
+                },
+                max_version.1,
+                architectures,
+            ));
         }
 
+        sources_architectures.sort_by_key(|(source, _, _)| source.clone());
         Ok(sources_architectures)
     }
 }
@@ -229,19 +238,14 @@ impl AsyncCommand for NMUVersionSkew<'_> {
         let sources = self.load_version_skew(self.options.suite)?;
 
         let mut wb_commands = Vec::new();
-        for (source, version, architectures) in sources {
-            let mut source = SourceSpecifier::new(&source);
+        for (source_package, binnmu_version, architectures) in sources {
+            let mut source = SourceSpecifier::new(&source_package.package);
             source.with_suite(self.options.suite);
             source.with_archive_architectures(architectures.as_ref());
-            let version_without_binnmu = version.clone().without_binnmu_version();
-            source.with_version(&version_without_binnmu);
+            source.with_version(&source_package.version);
 
             let mut binnmu = BinNMU::new(&source, "Rebuild to sync binNMU versions")?;
             binnmu.with_build_priority(self.options.build_priority);
-            let Some(binnmu_version) = version.binnmu_version() else {
-                error!("Skipping {}: package version has no binNMU.", source);
-                continue;
-            };
             binnmu.with_nmu_version(binnmu_version);
 
             wb_commands.push(binnmu.build());
