@@ -8,8 +8,14 @@ use std::{
 };
 
 use anyhow::Result;
+use assorted_debian_utils::{
+    archive::{Codename, SuiteOrCodename},
+    package::PackageName,
+};
 use serde::Deserialize;
 
+use crate::config::Cache;
+
 #[derive(Debug, Clone, Copy, Deserialize, PartialEq, Eq, PartialOrd, Ord)]
 #[serde(rename_all = "lowercase")]
 pub enum Severity {
@@ -51,7 +57,7 @@ impl Display for Severity {
 #[derive(Clone, Debug, Deserialize)]
 pub struct UDDBug {
     pub id: u32,
-    pub source: String,
+    pub source: PackageName,
     pub severity: Severity,
     pub title: String,
 }
@@ -59,11 +65,16 @@ pub struct UDDBug {
 #[derive(Default)]
 pub struct UDDBugs {
     bugs: Vec<UDDBug>,
-    source_index: HashMap<String, Vec<usize>>,
+    source_index: HashMap<PackageName, Vec<usize>>,
 }
 
 impl UDDBugs {
-    pub fn new(bugs: Vec<UDDBug>) -> Self {
+    pub fn load_for_codename(cache: &Cache, suite: SuiteOrCodename) -> Result<Self> {
+        let codename = Codename::from(suite);
+        load_bugs_from_reader(cache.get_cache_bufreader(format!("udd-ftbfs-bugs-{codename}.yaml"))?)
+    }
+
+    fn new(bugs: Vec<UDDBug>) -> Self {
         let mut udd_bugs = Self {
             bugs,
             ..Default::default()
@@ -80,14 +91,14 @@ impl UDDBugs {
         udd_bugs
     }
 
-    pub fn bugs_for_source(&self, source: &str) -> Option<Vec<&UDDBug>> {
+    pub fn bugs_for_source(&self, source: &PackageName) -> Option<Vec<&UDDBug>> {
         self.source_index
             .get(source)
             .map(|indices| indices.iter().map(|idx| &self.bugs[*idx]).collect())
     }
 }
 
-pub fn load_bugs_from_reader(reader: impl Read) -> Result<UDDBugs> {
+fn load_bugs_from_reader(reader: impl Read) -> Result<UDDBugs> {
     serde_yaml::from_reader(reader)
         .map_err(Into::into)
         .map(UDDBugs::new)
@@ -144,10 +155,16 @@ mod test {
     fn read_bugs() {
         let bugs = load_bugs_from_reader(TEST_DATA.as_bytes()).unwrap();
 
-        assert!(bugs.bugs_for_source("dmtcp").is_some());
-        assert!(bugs.bugs_for_source("zathura").is_none());
+        assert!(bugs.bugs_for_source(&"dmtcp".try_into().unwrap()).is_some());
+        assert!(
+            bugs.bugs_for_source(&"zathura".try_into().unwrap())
+                .is_none()
+        );
 
-        for bug in bugs.bugs_for_source("mutextrace").unwrap() {
+        for bug in bugs
+            .bugs_for_source(&"mutextrace".try_into().unwrap())
+            .unwrap()
+        {
             assert!(bug.severity >= Severity::Serious);
             // assert!(bug.severity.is_rc());
         }
